# -*- coding: utf-8 -*-
"""
/***************************************************************************
 DigiAgriApp
                                 A QGIS plugin
 Plugins to work with DigiAgriApp server
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                              -------------------
        begin                : 2023-02-23
        git sha              : $Format:%H$
        copyright            : (C) 2023 by Luca Delucchi
        email                : lucadeluge@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
import sys
import os
import json
import requests
import shapely
import tempfile
from datetime import datetime
from shapely.geometry import Point
from shapely.geometry import LineString
import numpy

from qgis.PyQt.QtCore import Qt
from qgis.PyQt.QtCore import QSettings
from qgis.PyQt.QtCore import QTranslator
from qgis.PyQt.QtCore import QCoreApplication
from qgis.PyQt.QtGui import QIcon
from qgis.PyQt.QtGui import QColor
from qgis.PyQt.QtWidgets import QAction
from qgis.PyQt.QtWidgets import QMenu
from qgis.PyQt.QtWidgets import QTableWidgetItem
from qgis.PyQt.QtWidgets import QComboBox
from qgis.PyQt.QtWidgets import QAbstractScrollArea
from qgis.PyQt.QtWidgets import QLineEdit
from qgis.PyQt.QtWidgets import QFileDialog
from qgis.PyQt.QtCore import QVariant

from qgis.core import QgsProject
from qgis.core import Qgis
from qgis.core import QgsVectorLayer
from qgis.core import QgsRasterLayer
from qgis.core import QgsJsonExporter
from qgis.core import QgsVectorFileWriter
from qgis.core import QgsGeometry
from qgis.core import QgsFields
from qgis.core import QgsField
from qgis.core import QgsFeature
from qgis.core import QgsJsonUtils
from qgis.core import QgsPointXY
from qgis.core import QgsSymbol
from qgis.core import QgsRendererCategory
from qgis.core import QgsCategorizedSymbolRenderer
from qgis.core import QgsSingleSymbolRenderer
from qgis.core import QgsLineSymbol
from qgis.core import QgsMarkerSymbol
from qgis.core import QgsFillSymbol
from qgis.core import QgsPalLayerSettings
from qgis.core import QgsTextFormat
from qgis.core import QgsVectorLayerSimpleLabeling
from qgis.core import QgsTextBufferSettings


## also import pyqtRemoveInputHook
from qgis.PyQt.QtCore import pyqtRemoveInputHook
import pdb

pyqtRemoveInputHook()
# pdb.set_trace()
## Initialize Qt resources from file resources.py
# from .resources_rc import *

from .functions import get_utm_srid_from_extent
from .functions import QuestionDialog
from .functions import createPlantPoleFeature
from .functions import ProgressBarDialog
# Import the code for the dialog
from .digiagriapp_dialog_import import DigiAgriAppImport
from .digiagriapp_dialog_settings import DigiAgriAppSettings
from .digiagriapp_dialog_about import DigiAgriAppAbout
from .digiagriapp_dialog_export_vector import DigiAgriAppExportVector
from .digiagriapp_dialog_plants import DigiAgriAppPlantCreation

sys.path.append(os.path.dirname(__file__))

FARM_URL = "{server}api/v1/farms/"
FARM_STATS_URL = FARM_URL + "{farmid}/stats/"
FIELD_URL = FARM_URL + "{farmid}/fields/"
SUBFIELD_URL = FIELD_URL + "{fieldid}/subfields/"
FIELD_ROW_URL = FIELD_URL + "{fieldid}/rows/"
SUBFIELD_ROW_URL = SUBFIELD_URL + "{subfieldid}/rows/"
PLANT_URL = FIELD_URL + "{fieldid}/plants/"
POLE_URL = FIELD_URL + "{fieldid}/poles/"
MATERIAL_URL = FARM_URL + "{farmid}/materials/"
FIELD_IMPORT = "{server}api/v1/{typ}-import"
PLANTS_CREATE = "{server}api/v1/plant-full-import/"


class DigiAgriApp:
    """QGIS Plugin Implementation."""

    @staticmethod
    def _request_error(req):
        """Return the error in a request"""
        if "detail" in req.keys():
            return req["detail"]
        else:
            return req

    def _refresh_table(self):
        """Refresh table"""
        self.import_dlg.tableWidget.setSizeAdjustPolicy(
            QAbstractScrollArea.AdjustToContents
        )
        self.import_dlg.tableWidget.resizeColumnsToContents()

    def _get_farm_stats(self):
        """Return statistics about farm"""
        farm_url = FARM_STATS_URL.format(
            server=self.server,
            farmid=self.farm_id
        )
        with requests.Session() as ses:
            try:
                req = ses.get(
                    farm_url,
                    headers={"Authorization": f"Token {self.token}"}
                )
            except Exception as er:
                self.iface.messageBar().pushMessage(
                self.tr("Error"),
                self.tr(f"Problem qetting the farms: {er}"),
                    level=Qgis.Success,
                )
                return False
            if not req.ok:
                try:
                    reason = self._request_error(req.json())
                except:
                    reason = req.text
                self.iface.messageBar().pushMessage(
                    self.tr("Error"),
                    self.tr(f"Problem to get farms; error: {reason}")
                )
                return False
        return req.json()["data"]

    def _get_fields(self):
        """Return available fields for farm"""
        fields_url = FIELD_URL.format(server=self.server, farmid=self.farm_id)
        print(fields_url)
        with requests.Session() as ses:
            req = ses.get(
                fields_url,
                headers={"Authorization": f"Token {self.token}"}
            )
            if not req.ok:
                reason = self._request_error(req.json())
                self.iface.messageBar().pushMessage(
                    self.tr("Error"),
                    self.tr(
                        f"Problem to get fields for farm with ID {self.farm_id}; error: {reason}"
                    ),
                )
                return False
            self.fields = req.json()
            self.fields_id = {}
            for field in self.fields:
                self.fields_id[field["id"]] = field["name"]
        print(self.fields_id)
        return True

    def _get_subfields(self, fieldid):
        """API call to get subfields for a field

        Args:
            fieldid (int): the ID of the field

        Returns:
            JSON: all the subfields for a field or an error
        """
        subfields_url = SUBFIELD_URL.format(
            server=self.server,
            farmid=self.farm_id,
            fieldid=fieldid
        )
        print(subfields_url)
        with requests.Session() as ses:
            req = ses.get(
                subfields_url,
                headers={"Authorization": f"Token {self.token}"}
            )
            if not req.ok:
                reason = self._request_error(req.json())
                self.iface.messageBar().pushMessage(
                    self.tr("Warning"),
                    self.tr(
                        f"Problem to get subfield for farm with ID {self.farm_id} with subfield with ID {fieldid}; error: {reason}"
                    ),
                )
                return False
        return req.json()

    def _get_rows(self, fieldid, subfieldid):
        """API call to get rows for a field or a subfield

        Args:
            fieldid (_type_): _description_
            subfieldid (_type_): _description_

        Returns:
            JSON: all the row for a field and subfile or an error
        """
        if subfieldid:
            rows_url = SUBFIELD_ROW_URL.format(
                server=self.server,
                farmid=self.farm_id,
                fieldid=fieldid,
                subfieldid=subfieldid
            )
        else:
            rows_url = FIELD_ROW_URL.format(
                server=self.server,
                farmid=self.farm_id,
                fieldid=fieldid
            )
        print(rows_url)
        with requests.Session() as ses:
            try:
                req = ses.get(
                    rows_url,
                    headers={"Authorization": f"Token {self.token}"}
                )
            except Exception as err:
                self.iface.messageBar().pushMessage(
                self.tr("Error"),
                self.tr(f"Problem qetting the farms: {err}"),
                    level=Qgis.Success,
                )
                return False
            if not req.ok:
                try:
                    reason = self._request_error(req.json())
                except:
                    reason = req.text
                self.iface.messageBar().pushMessage(
                    self.tr("Warning"),
                    self.tr(f"Problem to get rows; error: {reason}")
                )
                return False
        return req.json()


    def _get_farms(self):
        """Return available farms for user"""
        farms_url = FARM_URL.format(server=self.server)
        with requests.Session() as ses:
            try:
                req = ses.get(
                    farms_url,
                    headers={"Authorization": f"Token {self.token}"}
                )
            except Exception as er:
                self.iface.messageBar().pushMessage(
                self.tr("Error"),
                self.tr(f"Problem qetting the farms: {er}"),
                    level=Qgis.Success,
                )
                return False
            if not req.ok:
                try:
                    reason = self._request_error(req.json())
                except:
                    reason = req.text
                self.iface.messageBar().pushMessage(
                    self.tr("Error"),
                    self.tr(f"Problem to get farms; error: {reason}")
                )
                return False
            self.settings_dlg.farms_comboBox.clear()
            for farm in req.json():
                self.settings_dlg.farms_comboBox.addItem(
                    farm["name"],
                    farm["id"]
                )
        print(f"In _get_farms {self.farm}")
        if not self.farm:
            if self.settings.contains("/DigiAgriApp/farm"):
                self.farm = self.settings.value("/DigiAgriApp/farm", self.farm)
            if self.settings.contains("/DigiAgriApp/farm_id"):
                self.farm_id = self.settings.value("/DigiAgriApp/farm_id", self.farm_id)
        print(f"{self.farm} {self.farm_id}")
        if self.farm and self.farm_id:
            farmindex = self.settings_dlg.farms_comboBox.findData(self.farm_id)
            print(f"farm index {farmindex}")
            self.farm_stats = self._get_farm_stats()
            self.settings_dlg.farms_comboBox.setCurrentIndex(farmindex)
            self._get_fields()
        else:
            return False
        return True

    def _get_material(self, csv=False):
        """Return material in a QGIS table"""
        print("inside get material")
        pbd = ProgressBarDialog(
            parent=self.iface.mainWindow(),
            label=self.tr(f"Downloading material for farm {self.farm}")
        )
        if csv:
            mats_url = MATERIAL_URL.format(server=self.server, farmid=self.farm_id) + "?format=csv"
        else:
            mats_url = MATERIAL_URL.format(server=self.server, farmid=self.farm_id) + "?full=yes"
        with requests.Session() as ses:
            try:
                req = ses.get(
                    mats_url,
                    headers={"Authorization": f"Token {self.token}"}
                )
            except Exception as er:
                self.iface.messageBar().pushMessage(
                self.tr("Error"),
                self.tr(f"Problem qetting the farms: {er}"),
                    level=Qgis.Success,
                )
                return False
            if not req.ok:
                try:
                    reason = self._request_error(req.json())
                except:
                    reason = req.text
                self.iface.messageBar().pushMessage(
                    self.tr("Error"),
                    self.tr(f"Problem to get farms; error: {reason}")
                )
                return False
        if csv:
            csv_path = f"{tempfile.mkstemp()[1]}.csv"
            fil = open(csv_path, mode='w')
            fil.write(req.text)
            fil.close()
            pbd.setValue(50)
            csv_uri = f"file://{csv_path}?&delimiter=,&trimFields=yes&geomType=none"
            csv = QgsVectorLayer(csv_uri, 'material_csv', 'delimitedtext')
            QgsProject.instance().addMapLayer(csv)
            pbd.setValue(100)
        else:
            mats = req.json()
            total_fields = len(mats)
            n_fields = 1
            year = datetime.today().year
            minyear = year
            maxyear = year
            for mat in mats:
                try:
                    year = int(mat["start"].split("-")[0])
                    if year < minyear and year >= 1990:
                        minyear = year
                except:
                    pass
                self.material[mat["id"]] = mat
                pbd.setValue(int((n_fields / total_fields) * 100))
                n_fields += 1
            self.minmax_years = [minyear, maxyear]
        print("finish get material")
        return True



    def _check_server(self):
        """Check is server is save to settings"""
        if self.settings.contains("/DigiAgriApp/server"):
            self.settings_dlg.server_lineEdit.setText(
                self.settings.value("/DigiAgriApp/server")
            )
            self.server = self.settings.value("/DigiAgriApp/server")
            if not self.server.endswith("/"):
                self.server += "/"
            return True
        return False

    def _check_token(self):
        """Check if the token is saved in the settings"""
        if self.settings.contains("/DigiAgriApp/token"):
            self.token = self.settings.value("/DigiAgriApp/token")
            return True
        return False

    def _check_farm(self):
        """Check if the farm is saved in the setting"""
        print(f"start _check_farm {self.farm}")
        if self.settings.contains("/DigiAgriApp/farm"):
            self.farm = self.settings.value("/DigiAgriApp/farm")
        if self.settings.contains("/DigiAgriApp/farm_id"):
            self.farm_id = self.settings.value("/DigiAgriApp/farm_id")
        print(f"in _check_farm {self.farm}")
        if self.farm and self.farm_id:
            return True
        else:
            return False

    def _save_settings(self):
        """Running setting dialog"""
        # Run the dialog event loop
        result = self.settings_dlg.exec_()
        if result:
            user = self.settings_dlg.user_lineEdit.text()
            password = self.settings_dlg.password_lineEdit.text()
            self.server = self.settings_dlg.server_lineEdit.text()
            self.farm = self.settings_dlg.farms_comboBox.currentText()
            self.farm_id = self.settings_dlg.farms_comboBox.currentData()
            print(f"Farm {self.farm}")

            missing = []
            if not self.server:
                missing.append("server")
            else:
                if not self.server.endswith("/"):
                    self.server += "/"
                self.settings.setValue("/DigiAgriApp/server", self.server)
            if not user:
                missing.append("user")
            else:
                self.settings.setValue("/DigiAgriApp/user", user)
            if not password:
                missing.append("password")
            else:
                self.settings.setValue("/DigiAgriApp/password", password)
            if self.farm:
                self.settings.setValue("/DigiAgriApp/farm", self.farm)
                self.settings.setValue("/DigiAgriApp/farm_id", self.farm_id)
            if len(missing) != 0:
                self.iface.messageBar().pushMessage(
                    self.tr("Error"),
                    self.tr("Please set {} values".format(",".join(missing))),
                    level=Qgis.Critical,
                )

            user = self.settings.value("/DigiAgriApp/user")
            password = self.settings.value("/DigiAgriApp/password")
            loginurl = self.server + "api/token-auth/"
            try:
                req = requests.post(
                    loginurl,
                    data={
                        "username": user,
                        "password": password,
                    },
                    timeout=400,
                )
            except Exception as er:
                self.iface.messageBar().pushMessage(
                    self.tr("Error"),
                    self.tr(f"Problem requesting the token: {er}"),
                    level=Qgis.Success,
                )
                raise er
            try:
                self.token = req.json()["token"]
                self.settings.setValue("/DigiAgriApp/token", self.token)
                self.iface.messageBar().pushMessage(
                    self.tr("Success"),
                    self.tr("Token acquired correctly"),
                    level=Qgis.Success,
                )
            except:
                self.iface.messageBar().pushMessage(
                    self.tr("Error"),
                    self.tr(
                        "Problem to get the token, please check server, user and password"
                    ),
                    level=Qgis.Critical,
                )
                return False
            if self._check_token():
                print("dopo _check_token, before _get_farms")
                self._get_farms()
                self._get_material()

    def _working_server(self):
        """Function to check if everything is working properly with the server"""
        if not self._check_server():
            self.iface.messageBar().pushMessage(
                                self.tr("Error"),
                                self.tr("Impossible get the server url, please check the settings."),
                                level=Qgis.Critical,
                            )
            return False
        token_exist = self._check_token()
        if not token_exist:
            self.iface.messageBar().pushMessage(
                                self.tr("Error"),
                                self.tr("Impossible connect to the server, please check the settings."),
                                level=Qgis.Critical,
                            )
            return False
        return True

    def _get_file(self, dialog):
        """Select a file"""
        dlg = QFileDialog.getSaveFileName(None, 'inFile', options=QFileDialog.DontConfirmOverwrite)
        if dlg[0]:
            if dialog == "export":
                self.export_vector_dlg.lineEditOutput.setText(dlg[0])
            elif dialog == "plants":
                self.plants_dlg.lineEditOutput.setText(dlg[0])
        return True

    def _import_vector(self, layer, allfeats, import_url):
        fields = [field.name() for field in layer.fields()]
        layer_name = layer.name
        export = QgsJsonExporter(layer)
        try:
            fidind = fields.index("fid")
        except ValueError:
            fidind = None
        if fidind is not None:
            export.setExcludedAttributes([fidind])
        if allfeats:
            layer_json = export.exportFeatures(
                layer.getFeatures()
            )
        else:
            layer_json = export.exportFeatures(
                layer.selectedFeatures()
            )

        if import_url[-1] != "/":
            import_url += "/"
        print(import_url)
        print(layer_json)
        with requests.Session() as ses:
            req = ses.post(
                import_url,
                data=layer_json,
                headers={
                    "Authorization": f"Token {self.token}",
                    "Content-type": "application/json"
                },
            )
            print(req.status_code)
            print(req.text)
            if req.ok:
                self.iface.messageBar().pushMessage(
                    self.tr("Success"),
                    self.tr(f"Layer '{layer_name}' imported correctly"),
                    level=Qgis.Success,
                )
            else:
                reason = self._request_error(req.json())
                self.iface.messageBar().pushMessage(
                    self.tr("Warning"),
                    self.tr(
                        f"Layer '{layer_name}' not imported reason: {reason}"
                    ),
                    level=Qgis.Success,
                )

    def _export_field(self, file_output):
        pbd = ProgressBarDialog(
            parent=self.iface.mainWindow(),
            label=self.tr(f"Downloading fields for farm {self.farm}")
        )
        field_fields = QgsFields()
        field_fields.append(QgsField("id", QVariant.Int))
        field_fields.append(QgsField("farm", QVariant.Int))
        field_fields.append(QgsField("name", QVariant.String))
        field_fields.append(QgsField("note", QVariant.String))
        field_fields.append(QgsField("soil", QVariant.Int))
        field_fields.append(QgsField("active_materials", QVariant.String))
        field_fields.append(QgsField("last_treatment", QVariant.String))
        field_fields.append(QgsField("last_operation", QVariant.String))

        vlf = QgsVectorLayer("MultiPolygon", "digiagriapp_field", "memory")
        prf = vlf.dataProvider()
        prf.addAttributes(field_fields)
        vlf.updateFields()
        total_fields = len(self.fields)
        n_fields = 0
        for field in self.fields:
            feat = QgsJsonUtils.stringToFeatureList(
                json.dumps(field["geom"]),
                field_fields,
                None
            )[0]
            print("prima", field["id"], feat.id())
            #feat.setId(field["id"])
            print("dopo", field["id"], feat.id())
            try:
                active_materials = " | ".join(field["active_materials"])
            except:
                active_materials = None
            feat.setAttributes([
                field["id"],
                field["farm"],
                field["name"],
                field["note"],
                field["soil"],
                active_materials,
                field["last_treatment"],
                field["last_operation"],
            ])
            prf.addFeatures([feat])
            n_fields += 1
            pbd.setValue(int((n_fields / total_fields) * 100))
        self._write_vector(vlf, file_output)
        return True

    def _export_subfield(self, file_output):
        pbd = ProgressBarDialog(
            parent=self.iface.mainWindow(),
            label=self.tr(f"Downloading subfields for farm {self.farm}")
        )
        subfield_fields = QgsFields()
        subfield_fields.append(QgsField("id", QVariant.Int))
        subfield_fields.append(QgsField("field", QVariant.Int))
        subfield_fields.append(QgsField("name", QVariant.String))
        subfield_fields.append(QgsField("note", QVariant.String))
        subfield_fields.append(QgsField("active_materials", QVariant.String))
        subfield_fields.append(QgsField("last_treatment", QVariant.String))
        subfield_fields.append(QgsField("last_operation", QVariant.String))

        vls = QgsVectorLayer("Polygon", "digiagriapp_subfield", "memory")
        prs = vls.dataProvider()
        prs.addAttributes(subfield_fields)
        vls.updateFields()
        if not self.farm_stats:
            self.farm_stats = self._get_farm_stats()
        total_subfields = self.farm_stats["subfields_count"]
        n_subfields = 0
        for field in self.fields:
            subfields = self._get_subfields(field["id"])
            if not subfields:
                continue
            for subfield in subfields:
                feat = QgsJsonUtils.stringToFeatureList(
                    json.dumps(subfield["geom"]),
                    subfield_fields,
                    None
                )[0]
                try:
                    active_materials = " | ".join(subfield["active_materials"])
                except:
                    active_materials = None
                feat.setAttributes([
                    subfield["id"],
                    subfield["field"],
                    subfield["name"],
                    subfield["note"],
                    active_materials,
                    subfield["last_treatment"],
                    subfield["last_operation"],
                ])
                prs.addFeatures([feat])
                n_subfields += 1
                pbd.setValue(int((n_subfields / total_subfields) * 100))
        self._write_vector(vls, file_output)

    def _export_row(self, file_output):
        pbd = ProgressBarDialog(
            parent=self.iface.mainWindow(),
            label=self.tr(f"Downloading rows for farm {self.farm}")
        )
        row_fields = QgsFields()
        row_fields.append(QgsField("id", QVariant.Int))
        row_fields.append(QgsField("field", QVariant.Int))
        row_fields.append(QgsField("subfield", QVariant.Int))
        row_fields.append(QgsField("name", QVariant.String))
        row_fields.append(QgsField("note", QVariant.String))
        row_fields.append(QgsField("active_materials", QVariant.String))
        row_fields.append(QgsField("last_treatment", QVariant.String))
        row_fields.append(QgsField("last_operation", QVariant.String))

        vlr = QgsVectorLayer("LineString", "digiagriapp_row", "memory")
        prr = vlr.dataProvider()
        prr.addAttributes(row_fields)
        vlr.updateFields()
        if not self.farm_stats:
            self.farm_stats = self._get_farm_stats()
        total_rows = self.farm_stats["rows_count"]
        n_rows = 0
        for fieldid, fieldname in self.fields_id.items():
            rows = self._get_rows(fieldid, None)
            if not rows:
                continue
            for row in rows:
                feat = QgsJsonUtils.stringToFeatureList(
                    json.dumps(row["geom"]),
                    row_fields,
                    None
                )[0]
                try:
                    active_materials = " | ".join(row["active_materials"])
                except:
                    active_materials = None
                feat.setAttributes([
                    row["id"],
                    row["field"],
                    row["subfield"],
                    row["name"],
                    row["note"],
                    active_materials,
                    row["last_treatment"],
                    row["last_operation"],
                ])
                prr.addFeatures([feat])
                n_rows += 1
                pbd.setValue(int((n_rows / total_rows) * 100))
        self._write_vector(vlr, file_output)

    def _export_plant(self, file_output):
        pbd = ProgressBarDialog(
            parent=self.iface.mainWindow(),
            label=self.tr(f"Downloading plants for farm {self.farm}")
        )
        plant_fields = QgsFields()
        plant_fields.append(QgsField("id", QVariant.Int))
        plant_fields.append(QgsField("field", QVariant.Int))
        plant_fields.append(QgsField("subfield", QVariant.Int))
        plant_fields.append(QgsField("row", QVariant.Int))
        plant_fields.append(QgsField("name", QVariant.String))
        plant_fields.append(QgsField("block", QVariant.Int))
        plant_fields.append(QgsField("position", QVariant.Int))
        plant_fields.append(QgsField("tag", QVariant.String))
        plant_fields.append(QgsField("note", QVariant.String))
        plant_fields.append(QgsField("active_materials", QVariant.String))
        plant_fields.append(QgsField("last_treatment", QVariant.String))
        plant_fields.append(QgsField("last_operation", QVariant.String))

        vlp = QgsVectorLayer("Point", "digiagriapp_plant", "memory")
        prp = vlp.dataProvider()
        prp.addAttributes(plant_fields)
        vlp.updateFields()
        if not self.farm_stats:
            self.farm_stats = self._get_farm_stats()
        total_plants = self.farm_stats["plants_count"]
        n_plants = 0
        for fieldid, fieldname in self.fields_id.items():
            plants_url = PLANT_URL.format(
                server=self.server,
                farmid=self.farm_id,
                fieldid=fieldid
            )
            with requests.Session() as ses:
                req = ses.get(
                    plants_url,
                    headers={"Authorization": f"Token {self.token}"}
                )
                if not req.ok:
                    reason = self._request_error(req.json())
                    self.iface.messageBar().pushMessage(
                        self.tr("Warning"),
                        self.tr(
                            f"Problem to get plants for farm with ID {self.farm_id} with field with ID {fieldid}; error: {reason}"
                        ),
                    )
                    continue
                plants = req.json()
                for plant in plants:
                    feat = QgsJsonUtils.stringToFeatureList(
                        json.dumps(plant["geom"]),
                        plant_fields,
                        None
                    )[0]
                    #feat.setId(plant["id"])
                    try:
                        active_materials = " | ".join(plant["active_materials"])
                    except:
                        active_materials = None
                    feat.setAttributes([
                        plant["id"],
                        plant["field"],
                        plant["subfield"],
                        plant["row"],
                        plant["name"],
                        plant["block"],
                        plant["position"],
                        plant["tag"],
                        plant["note"],
                        active_materials,
                        plant["last_treatment"],
                        plant["last_operation"],
                    ])
                    prp.addFeatures([feat])
                    n_plants += 1
                    pbd.setValue(int((n_plants / total_plants) * 100))
        self._write_vector(vlp, file_output)

    def _export_pole(self, file_output):
        """Function to export infrestructure pole"""
        pbd = ProgressBarDialog(
            parent=self.iface.mainWindow(),
            label=self.tr(f"Downloading poles for farm {self.farm}")
        )
        pole_fields = QgsFields()
        pole_fields.append(QgsField("id", QVariant.Int))
        pole_fields.append(QgsField("farm", QVariant.Int))
        pole_fields.append(QgsField("field", QVariant.Int))
        pole_fields.append(QgsField("subfield", QVariant.Int))
        pole_fields.append(QgsField("row", QVariant.Int))
        pole_fields.append(QgsField("note", QVariant.String))

        vlp = QgsVectorLayer("Point", "digiagriapp_pole", "memory")
        prp = vlp.dataProvider()
        prp.addAttributes(pole_fields)
        vlp.updateFields()
        if not self.farm_stats:
            self.farm_stats = self._get_farm_stats()
        total_poles = self.farm_stats["poles_count"]
        n_poles = 0
        for fieldid, fieldname in self.fields_id.items():
            poles_url = POLE_URL.format(
                server=self.server,
                farmid=self.farm_id,
                fieldid=fieldid
            )
            print(poles_url)
            with requests.Session() as ses:
                req = ses.get(
                    poles_url,
                    headers={"Authorization": f"Token {self.token}"}
                )
                if not req.ok:
                    reason = self._request_error(req.json())
                    self.iface.messageBar().pushMessage(
                        self.tr("Warning"),
                        self.tr(
                            f"Problem to get poles for farm with ID {self.farm_id} with field with ID {fieldid}; error: {reason}"
                        ),
                    )
                    continue
                poles = req.json()
                for plant in poles:
                    feat = QgsJsonUtils.stringToFeatureList(
                        json.dumps(plant["geom"]),
                        pole_fields,
                        None
                    )[0]
                    #feat.setId(plant["id"])
                    feat.setAttributes([
                        plant["id"],
                        plant["farm"],
                        plant["field"],
                        plant["subfield"],
                        plant["row"],
                        plant["note"],
                    ])
                    prp.addFeatures([feat])
                    n_poles += 1
                    pbd.setValue(int((n_poles / total_poles) * 100))
        self._write_vector(vlp, file_output)

    def _reload_layer(self, infile, layername):
        """Function to reload a layer in QGIS canva

        Args:
            infile (str): the path to gpkg file
            layername (str): the name of layer to load, it can be field, subfield, row, plant
        """
        layers = QgsProject.instance().mapLayersByName(f"digiagriapp_{layername}")
        #csv = QgsProject.instance().mapLayersByName("material_csv")[0]
        layer = None
        if len(layers) == 0:
            layer = self.iface.addVectorLayer(f"{infile}|layername=digiagriapp_{layername}", f"digiagriapp_{layername}", "ogr")
        elif len(layers) == 1:
            layer = layers[0]
            layer.reload()
        else:
            QgsProject.instance().reloadAllLayers()
        return layer

    def _create_points(self, line, info, row_name, field, subfield, rowid, check_poles = False):
        """Function to create points along the line"""
        linecoors = line.coords
        npoints = info["pole_number"]
        plants_block = info["plants_block"]
        first = linecoors[0]
        last = linecoors[-1]
        # work only with shapely > 2
        if shapely.__version__ > "2.0.0":
            if last < first:
                line = line.reverse()
                linecoors = line.coords
        else:
            if last < first:
                linelist = list(linecoors)
                linelist.reverse()
                line = LineString(linelist)
                linecoors = line.coords
        # create qgis vector for the line
        row = QgsVectorLayer("LineString", f"{row_name}_row", "memory")
        row_prov = row.dataProvider()
        row_prov.addAttributes([QgsField("name", QVariant.String)])
        row.updateFields()
        row_feat = QgsFeature()
        row_feat.setGeometry(QgsGeometry.fromWkt(line.wkt))
        row_feat.setAttributes([row_name])
        print(row_feat.isValid(), row_prov.featureCount())
        if not row_prov.addFeatures([row_feat]):
            print(f"{row_feat} not added")
        row.updateExtents()
        QgsProject.instance().addMapLayer(row)
        line_coords = []
        for coor in linecoors:
            line_coords.append(coor)
        usepoints = npoints - 1

        # epsg = get_utm_srid_from_extent(line.bounds)
        # utm = f"EPSG:{epsg}"
        # wgs84 = QgsCoordinateReferenceSystem("EPSG:4326")
        # transformContext = QgsProject.instance().transformContext()
        # xform = QgsCoordinateTransform(wgs84, utm, transformContext)
        # utm_line = xform.transform(line)

        # epsg = utm_from_extent(line.bounds)
        # wgs84 = pyproj.CRS("EPSG:4326")
        # project = pyproj.Transformer.from_crs(wgs84, utm, always_xy=True).transform
        # utm_line = geotransform(project, line)

        # create the point of poles interpolation the line
        polepoints = [line.interpolate((i/usepoints), normalized=True) for i in range(1, usepoints)]
        segment_num = 0
        final_points = [line_coords[0]]
        # create vector for points
        plants = QgsVectorLayer("Point", f"{row_name}_plants", "memory")
        plants_prov = plants.dataProvider()
        plants_prov.addAttributes([
            QgsField("type", QVariant.String),
            QgsField("material", QVariant.Int),
            QgsField("label", QVariant.String),
            QgsField("theory", QVariant.Int),
            QgsField("evidence", QVariant.Int),
            QgsField("farm", QVariant.Int),
            QgsField("field", QVariant.Int),
            QgsField("subfield", QVariant.Int),
            QgsField("row", QVariant.Int),
        ])
        plants.updateFields()

        totalpoints = 1
        plantsnumb = 0
        # check if first is pole or plant
        if info["first"] == "pole":
            plant_feat = QgsFeature()
            plant_feat.setGeometry(
                QgsGeometry.fromPointXY(
                    QgsPointXY(line_coords[0][0], line_coords[0][1])
                )
            )
            plant_feat.setAttributes(
                ["pole", None, None, None, None, self.farm_id, field, subfield, rowid]
            )
            print(plant_feat.isValid(), plants_prov.featureCount())
            if not plants_prov.addFeatures([plant_feat]):
                print(f"{plant_feat} not added")
            #
            norm = numpy.random.normal(plants_block[1:-1])
            minor_value = round(norm.mean() - norm.std() -1)
            next_poi = None
            secondtolast_poi = None
            final_line  = None
            last_poi = Point(line_coords[-1])
            # first segment is for shorter block
            if plants_block[0] < minor_value and not check_poles:
                thisline = LineString([line_coords[0], list(polepoints[0].coords)[0]])
                leng = thisline.length
                space = leng / max(plants_block)
                maxleng = space * plants_block[0]
                next_poi = thisline.interpolate(maxleng)
            # last segment
            if info["last"] == "plant":
                if next_poi:
                    linereverse = LineString([line_coords[-1], next_poi])
                else:
                    linereverse = LineString([line_coords[-1], line_coords[0]])
                last_poi = linereverse.interpolate(0.0000045)
                if plants_block[-2] < minor_value and not check_poles:
                    final_line = LineString([polepoints[-1].coords[0], last_poi.coords[0]])
            else:
                if plants_block[-1] < minor_value and not check_poles:
                    final_line = LineString([polepoints[-1].coords[0], line_coords[-1]])
            if final_line:
                leng = final_line.length
                space = leng / max(plants_block)
                maxleng = space * plants_block[-1]
                if shapely.__version__ > "2.0.0":
                    linereverse = final_line.reverse()
                else:
                    linereverse = LineString([line_coords[-1], list(polepoints[-1].coords)[0]])
                secondtolast_poi = linereverse.interpolate(maxleng)
            finalpolepoints = [Point(line.coords[0])]
            if next_poi and secondtolast_poi:
                newline = LineString([next_poi, secondtolast_poi])
                newusepoints = usepoints - 2
                newpolepoints = [newline.interpolate((i/newusepoints), normalized=True) for i in range(1, newusepoints)]
                finalpolepoints.append(next_poi)
                finalpolepoints.extend(newpolepoints)
                finalpolepoints.append(secondtolast_poi)
            elif next_poi and not secondtolast_poi:
                newline = LineString([next_poi, line_coords[-1]])
                newusepoints = usepoints - 1
                newpolepoints = [newline.interpolate((i/newusepoints), normalized=True) for i in range(1, newusepoints)]
                finalpolepoints.append(next_poi)
                finalpolepoints.extend(newpolepoints)
            elif secondtolast_poi and not next_poi:
                newline = LineString([line_coords[0], secondtolast_poi])
                newusepoints = usepoints - 1
                newpolepoints = [newline.interpolate((i/newusepoints), normalized=True) for i in range(1, newusepoints)]
                finalpolepoints.extend(newpolepoints)
                finalpolepoints.append(secondtolast_poi)
            else:
                finalpolepoints.extend(polepoints)
            finalpolepoints.append(last_poi)
        # first is a plant
        else:
            next_poi = None
            distance = 0
            for npoi in range(info["plants_block"][0]):
                plant_info = info["points"][npoi]
                if plant_info["material"] in ["F", "f"]:
                    # distance 60cm
                    distance += 0.6 / 111139
                elif plant_info["material"] in ["", "p", "P"]:
                    raise ValueError("This plant seem to be a pole")
                else:
                    if "distance" in plant_info.keys():
                        distance += plant_info["distance"] / 111139
                    else:
                        distance += 0.6 / 111139
                if next_poi is None:
                    geom = QgsGeometry.fromPointXY(
                        QgsPointXY(line_coords[0][0], line_coords[0][1])
                    )
                else:
                    nextcoords = list(next_poi.coords)[0]
                    geom = QgsGeometry.fromPointXY(
                        QgsPointXY(nextcoords[0], nextcoords[1])
                    )

                plant_info["type"] = "plant"
                plant_feat = createPlantPoleFeature(
                    plant_info,
                    geom,
                    self.farm_id,
                    field,
                    subfield,
                    rowid
                )
                print(plant_feat.isValid(), plants_prov.featureCount())
                if plant_feat.isValid():
                    if not plants_prov.addFeature(plant_feat):
                        print(f"{plant_feat} not added")


                # move the first pole 50cm (second row in the csv file)
                next_poi = line.interpolate(distance)
                plantsnumb += 1
            secondtolast_poi = None
            # if last point not pole
            finalpolepoints = [next_poi]
            if info["last"] == "plant":
                final_distance = 0
                for npoi in range(info["plants_block"][-1]):
                    if info["points"][npoi]["material"] in ["F", "f"]:
                        material = None
                        # distance 60cm
                        final_distance += 0.6 / 111139
                    elif info["points"][npoi]["material"] in ["", "p", "P"]:
                        raise ValueError("This plant seem to be a pole")
                    else:
                        if "distance" in plant_info.keys():
                            final_distance += plant_info["distance"] / 111139
                        else:
                            final_distance += 0.6 / 111139
                    linereverse = LineString([line_coords[-1], next_poi])
                    secondtolast_poi = linereverse.interpolate(distance)
            if secondtolast_poi is not None:
                newline = LineString([next_poi, secondtolast_poi])
                newpolepoints = [newline.interpolate((i/usepoints), normalized=True) for i in range(1, usepoints)]
                finalpolepoints.extend(newpolepoints)
                finalpolepoints.append(secondtolast_poi)
            else:
                newline = LineString([next_poi, line_coords[-1]])
                newpolepoints = [newline.interpolate((i/usepoints), normalized=True) for i in range(1, usepoints)]
                finalpolepoints.extend(newpolepoints)
                finalpolepoints.append(Point(line_coords[-1]))

        # for each pole create the block line and interpolate all the plants of that block
        for poi in finalpolepoints[1:]:
            poicoords = list(poi.coords)[0]
            if segment_num == 0:
                polecoords = list(finalpolepoints[0].coords[0])
                thisline = LineString([polecoords, poicoords])
                # first was already added before
                if info["first"] == "plant":
                    final_points.append(poicoords)
                    plant_feat = QgsFeature()
                    plant_feat.setGeometry(
                        QgsGeometry.fromPointXY(
                            QgsPointXY(polecoords[0], polecoords[1])
                        )
                    )
                    plant_feat.setAttributes(
                        ["pole", None, None, None, None, self.farm_id, field, subfield, rowid]
                    )
                    if not plants_prov.addFeatures([plant_feat]):
                        print(f"{plant_feat} not added")
                    totalpoints += 1
                    #segment_num += 1

            print(segment_num, npoints)
            if segment_num == npoints:
                if info["last"] == "plant":
                    continue
            else:
                thisline = LineString([finalpolepoints[segment_num], poicoords])
            if info["first"] == "plant":
                thispoints = [thisline.interpolate((i/(plants_block[segment_num + 1] + 1)), normalized=True) for i in range(1, plants_block[segment_num +1] + 1)]
            else:
                thispoints = [thisline.interpolate((i/(plants_block[segment_num] + 1)), normalized=True) for i in range(1, plants_block[segment_num] + 1)]
            print(f"Number of points {thispoints}")

            #for tpoi in thispoints:
            jump_val = 0
            for tpoi_index in range(len(thispoints)):
                right_index = tpoi_index + jump_val
                if right_index < len(thispoints):
                    tpoi = thispoints[right_index]
                else:
                    continue
                tpoi_coord = tuple(tpoi.coords)[0]
                final_points.append(tpoi_coord)
                geom = QgsGeometry.fromPointXY(
                        QgsPointXY(tpoi_coord[0], tpoi_coord[1])
                    )
                if plantsnumb < len(info["points"]):
                    plant_info = info["points"][plantsnumb]
                else:
                    continue
                plant_info["type"] = "plant"
                plant_feat = createPlantPoleFeature(
                    plant_info,
                    geom,
                    self.farm_id,
                    field,
                    subfield,
                    rowid
                )

                if plant_feat.isValid():
                    print(plant_feat.isValid(), plants_prov.featureCount())
                    if not plants_prov.addFeature(plant_feat):
                        print(f"{plant_feat} not added")
                    totalpoints += 1
                    plantsnumb += 1
                else:
                    raise ValueError("Plant not valid")
                if "multiplant" in plant_info.keys():
                    if plant_info["multiplant"] and (right_index + 1) < len(thispoints):
                        plant_info = info["points"][plantsnumb]
                        if plant_info["material"] in ["f", "F"] or (plant_info["material"] not in ["", "p", "P"] and plant_info["multiplant"]):
                            jump_val += 1
                            next_poi = thispoints[right_index + 1]
                            next_coords = tuple(next_poi.coords)[0]
                            next_line = LineString([tpoi_coord, next_coords])
                            final_next_poi = next_line.interpolate(0.1 / 111139)
                            final_next_coords = tuple(final_next_poi.coords)[0]
                            next_geom = QgsGeometry.fromPointXY(
                                QgsPointXY(final_next_coords[0], final_next_coords[1])
                            )

                            plant_info["type"] = "plant"
                            plant_feat = createPlantPoleFeature(
                                plant_info,
                                next_geom,
                                self.farm_id,
                                field,
                                subfield,
                                rowid
                            )

                            if plant_feat.isValid():
                                print(plant_feat.isValid(), plants_prov.featureCount())
                                if not plants_prov.addFeature(plant_feat):
                                    print(f"{plant_feat} not added")
                                totalpoints += 1
                                plantsnumb += 1

            print(plants_prov.featureCount())
            final_points.append(poicoords)
            plant_feat = QgsFeature()
            plant_feat.setGeometry(
                QgsGeometry.fromPointXY(
                    QgsPointXY(poicoords[0], poicoords[1])
                )
            )
            plant_feat.setAttributes(
                ["pole", None, None, None, None, self.farm_id, field, subfield, rowid]
            )
            if not plants_prov.addFeatures([plant_feat]):
                print(f"{plant_feat} not added")
            totalpoints += 1
            segment_num += 1
        # if there is a last block of plant after last pole
        if info["last"] == "plant":
            linereverse = LineString([line_coords[-1], finalpolepoints[-1]])
            plantpoints = [linereverse.interpolate((i/info["plants_block"][-1]), normalized=True) for i in range(1, info["plants_block"][-1])]
            for plantpoi in plantpoints:
                plant_info =  info["points"][plantsnumb]
                plant_info["type"] = "plant"
                plant_coords = list(plantpoi.coords)[0]
                geom = QgsGeometry.fromPointXY(
                    QgsPointXY(plant_coords[0], plant_coords[1])
                )
                plant_feat = createPlantPoleFeature(
                    plant_info,
                    geom,
                    self.farm_id,
                    field,
                    subfield,
                    rowid
                )
                if plant_feat.isValid():
                    print(plant_feat.isValid(), plants_prov.featureCount())
                    if not plants_prov.addFeatures([plant_feat]):
                        print(f"{plant_feat} not added")
                totalpoints += 1
                plantsnumb += 1
            # add last plant
            if plantsnumb < len(info["points"]):
                plant_info =  info["points"][plantsnumb]
                plant_info["type"] = "plant"
                geom = QgsGeometry.fromPointXY(
                    QgsPointXY(line_coords[-1][0], line_coords[-1][1])
                )
                plant_feat = createPlantPoleFeature(
                    plant_info,
                    geom,
                    self.farm_id,
                    field,
                    subfield,
                    rowid
                )
                if plant_feat.isValid():
                    print(plant_feat.isValid(), plants_prov.featureCount())
                    if not plants_prov.addFeatures([plant_feat]):
                        print(f"{plant_feat} not added")
                totalpoints += 1
                plantsnumb += 1
        plants.updateExtents()
        QgsProject.instance().addMapLayer(plants)
        # set style
        # TODO improve style getting info from material
        classes = {"plant": ("#00FF00", "Plant"),
                   "pole": ("#808080", "Pole")}
        categories = []
        # Iterate through the dictionary
        for classes, (color, label) in classes.items():
            # Automatically set symbols based on layer's geometry
            symbol = QgsSymbol.defaultSymbol(plants.geometryType())
            # Set colour
            symbol.setColor(QColor(color))
            # Set symbol with value = 0 to be transparent
            if classes == "0":
                symbol.setOpacity(0)
            # Set the renderer properties
            category = QgsRendererCategory(classes, symbol, label)
            categories.append(category)

        # Field name
        expression = "type"
        # Set the categorized renderer
        renderer = QgsCategorizedSymbolRenderer(expression, categories)
        plants.setRenderer(renderer)
        # Refresh layer
        plants.triggerRepaint()

        return plants

    def _set_single_style(self, layer, color):
                # Set the categorized renderer
        symbol = QgsSymbol.defaultSymbol(layer.geometryType())
        symbol.setColor(QColor(color))
        if isinstance(symbol, QgsLineSymbol):
            symbol.setWidth(1)
        elif isinstance(symbol, QgsMarkerSymbol):
            symbol.setSize(2)
        elif isinstance(symbol, QgsMarkerSymbol):
            symbol.setOpacity(0.6)
        renderer = QgsSingleSymbolRenderer(symbol)
        layer.setRenderer(renderer)
        if "_row" not in layer.name() and "_pole" not in layer.name():
            label_settings = QgsPalLayerSettings()
            #label_settings.drawBackground = True
            if "_plant" not in layer.name():
                label_settings.fieldName = 'name'
            else:
                # TODO set theory label
                label_settings.fieldName = 'name'
            text_format = QgsTextFormat()
            text_buffer = QgsTextBufferSettings()
            text_buffer.setColor(QColor("#FFFFFF"))
            text_buffer.setSize(1)
            text_buffer.setEnabled(True)
            text_format.setBuffer(text_buffer)
            label_settings.setFormat(text_format)
            layer.setLabeling(QgsVectorLayerSimpleLabeling(label_settings))
            layer.setLabelsEnabled(True)
        # Refresh layer
        layer.triggerRepaint()
        return True


    def __init__(self, iface):
        """Constructor.

        :param iface: An interface instance that will be passed to this class
            which provides the hook by which you can manipulate the QGIS
            application at run time.
        :type iface: QgsInterface
        """
        # Save reference to the QGIS interface
        self.iface = iface
        # initialize plugin directory
        self.plugin_dir = os.path.dirname(__file__)
        # settings to save server, user, password
        self.settings = QSettings()
        # initialize locale
        locale = self.settings.value("locale/userLocale")#[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            "i18n",
            f"DigiAgriApp_{locale}.qm"
        )

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)
            QCoreApplication.installTranslator(self.translator)

        # settings useful variable
        self.server = None
        self.token = None
        self.farm = None
        self.farm_id = None
        self.farm_stats = None
        self.fields = None
        self.fields_id = {}
        self.layers_list = {}
        self.actions = []
        self.menu = None
        self.data_import = None
        self.material = {}
        self.minmax_years = []

        # Check if plugin was started the first time in current QGIS session
        # Must be set in initGui() to survive plugin reloads

        self.import_dlg = DigiAgriAppImport()
        self.import_dlg.button_box.accepted.connect(self.import_data)
        self.import_dlg.button_box.rejected.connect(self.import_dlg.close)
        self.about_dlg = DigiAgriAppAbout()
        self.settings_dlg = DigiAgriAppSettings()
        self.settings_dlg.savebutton.setToolTip("To be sure click twice")
        self.settings_dlg.savebutton.clicked.connect(self._save_settings)
        self.settings_dlg.closebutton.clicked.connect(self.settings_dlg.close)

        self.export_vector_dlg = DigiAgriAppExportVector()
        #self.export_vector_dlg.button_box.accepted.connect(self.export_vector)
        self.export_vector_dlg.button_box.rejected.connect(
            self.export_vector_dlg.close
        )
        self.export_vector_dlg.pushButton.clicked.connect(
            lambda: self._get_file("export")
        )

        self.plants_dlg = DigiAgriAppPlantCreation()
        self.plants_dlg.button_box.accepted.connect(self.plant_creation)
        self.plants_dlg.button_box.rejected.connect(self.plants_dlg.close)
        self.plants_dlg.comboBoxField.currentIndexChanged.connect(self.populate_subfields)
        self.plants_dlg.comboBoxSubfield.currentIndexChanged.connect(self.populate_rows)
        self.plants_dlg.pushButton.clicked.connect(lambda: self._get_file("plants"))

    # noinspection PyMethodMayBeStatic
    def tr(self, message):
        """Get the translation for a string using Qt translation API.

        We implement this ourselves since we do not inherit QObject.

        :param message: String for translation.
        :type message: str, QString

        :returns: Translated version of message.
        :rtype: QString
        """
        # noinspection PyTypeChecker,PyArgumentList,PyCallByClass
        return QCoreApplication.translate("DigiAgriApp", message)

    def build_menu(self):
        """Function to create the menu"""
        # import menu
        icon_import_path = self.plugin_dir + "/icons/icon_import.svg"
        self.import_menu = QMenu(self.tr("Import"), self.iface.mainWindow())
        self.import_menu.setIcon(QIcon(icon_import_path))
        ## vector
        icon_import_vector_path = self.plugin_dir + "/icons/icon_import_vector.svg"
        self.import_vector_act = QAction(
            QIcon(icon_import_vector_path),
            self.tr("Vector"),
            self.iface.mainWindow()
        )
        self.import_vector_act.triggered.connect(self.show_import_dialog)
        self.import_menu.addAction(self.import_vector_act)
        ## create plants
        icon_create_plant_path = self.plugin_dir + "/icons/icon_import_vector.svg"
        self.create_plant_act = QAction(
            QIcon(icon_create_plant_path),
            self.tr("Create plants from CSV file"),
            self.iface.mainWindow()
        )
        self.create_plant_act.triggered.connect(self.show_create_plant_dialog)
        self.import_menu.addAction(self.create_plant_act)
        ## raster
        icon_import_raster_path = self.plugin_dir + "/icons/icon_import_raster.svg"
        self.import_raster_act = QAction(
            QIcon(icon_import_raster_path),
            self.tr("Raster"),
            self.iface.mainWindow()
        )
        self.import_raster_act.triggered.connect(self.show_import_dialog)
        self.import_menu.addAction(self.import_raster_act)
        self.menu.addMenu(self.import_menu)

        # export menu
        icon_export_path = self.plugin_dir + "/icons/icon_export.svg"
        self.export_menu = QMenu(self.tr("Export"), self.iface.mainWindow())
        self.export_menu.setIcon(QIcon(icon_export_path))
        ## vector
        icon_export_vector_path = self.plugin_dir + "/icons/icon_export_vector.svg"
        self.export_vector_act = QAction(
            QIcon(icon_export_vector_path),
            self.tr("Vector"),
            self.iface.mainWindow()
        )
        self.export_vector_act.triggered.connect(self.show_export_vector_dialog)
        self.export_menu.addAction(self.export_vector_act)
        ## raster
        icon_export_raster_path = self.plugin_dir + "/icons/icon_export_raster.svg"
        self.export_raster_act = QAction(
            QIcon(icon_export_raster_path),
            self.tr("Raster"),
            self.iface.mainWindow()
        )
        self.export_raster_act.setEnabled(False)
        #self.export_raster_act.triggered.connect(self.show_export_raster_dialog)
        self.export_menu.addAction(self.export_raster_act)
        self.menu.addMenu(self.export_menu)

        # settings
        icon_settings_path = self.plugin_dir + "/icons/icon_settings.svg"
        self.settings_act = QAction(
            QIcon(icon_settings_path),
            self.tr("Settings"),
            self.iface.mainWindow()
        )
        self.actions.append(self.settings_act)
        self.settings_act.triggered.connect(self.show_settings_dialog)
        self.menu.addAction(self.settings_act)

        # about
        icon_about_path = self.plugin_dir + "/icons/icon_about.svg"
        self.about_act = QAction(
            QIcon(icon_about_path),
            self.tr("About"),
            self.iface.mainWindow()
        )
        self.actions.append(self.about_act)
        self.about_act.triggered.connect(self.about_dlg.show)
        self.menu.addAction(self.about_act)

    def add_action(
        self,
        icon_path,
        text,
        callback,
        enabled_flag=True,
        add_to_menu=True,
        add_to_toolbar=True,
        status_tip=None,
        whats_this=None,
        parent=None,
    ):
        """Add a toolbar icon to the toolbar.

        :param icon_path: Path to the icon for this action. Can be a resource
            path (e.g. ':/plugins/foo/bar.png') or a normal file system path.
        :type icon_path: str

        :param text: Text that should be shown in menu items for this action.
        :type text: str

        :param callback: Function to be called when the action is triggered.
        :type callback: function

        :param enabled_flag: A flag indicating if the action should be enabled
            by default. Defaults to True.
        :type enabled_flag: bool

        :param add_to_menu: Flag indicating whether the action should also
            be added to the menu. Defaults to True.
        :type add_to_menu: bool

        :param add_to_toolbar: Flag indicating whether the action should also
            be added to the toolbar. Defaults to True.
        :type add_to_toolbar: bool

        :param status_tip: Optional text to show in a popup when mouse pointer
            hovers over the action.
        :type status_tip: str

        :param parent: Parent widget for the new action. Defaults None.
        :type parent: QWidget

        :param whats_this: Optional text to show in the status bar when the
            mouse pointer hovers over the action.

        :returns: The action that was created. Note that the action is also
            added to self.actions list.
        :rtype: QAction
        """

        icon = QIcon(icon_path)
        action = QAction(icon, text, parent)
        action.triggered.connect(callback)
        action.setEnabled(enabled_flag)

        if status_tip is not None:
            action.setStatusTip(status_tip)

        if whats_this is not None:
            action.setWhatsThis(whats_this)

        if add_to_toolbar:
            # Adds plugin icon to Plugins toolbar
            self.iface.addToolBarIcon(action)

        if add_to_menu:
            self.iface.addPluginToWebMenu(None, action)
        self.actions.append(action)

        return action

    def initGui(self):
        """Create the menu entries and toolbar icons inside the QGIS GUI."""

        icon_path = self.plugin_dir + "/icons/icon.svg"
        self.menu = QMenu(self.tr("DigiAgriApp"))
        self.menu.setIcon(QIcon(icon_path))
        # self.add_action(
        #     icon_path,
        #     text=self.tr("DigiAgriApp"),
        #     callback=self.run,
        #     parent=self.iface.mainWindow(),
        # )

        # need workaround for WebMenu for QuickMapService
        _temp_act = QAction('temp', self.iface.mainWindow())
        self.iface.addPluginToWebMenu("_tmp", _temp_act)
        self.iface.webMenu().addMenu(self.menu)
        self.iface.removePluginWebMenu("_tmp", _temp_act)

        self.build_menu()
        self.iface.webMenu().addMenu(self.menu)

    def unload(self):
        """Removes the plugin menu item and icon from QGIS GUI."""
        if self.menu:
            self.iface.webMenu().removeAction(self.menu.menuAction())
            self.iface.addLayerMenu().removeAction(self.menu.menuAction())
        for action in self.actions:
            self.iface.webMenu().removeAction(action)
            self.iface.removeWebToolBarIcon(action)
            self.iface.removePluginWebMenu(self.tr("&DigiAgriApp"), action)
            self.iface.removeToolBarIcon(action)

    def show_settings_dialog(self):
        """Show settings dialog"""
        self._check_server()
        if self.settings.contains("/DigiAgriApp/user"):
            self.settings_dlg.user_lineEdit.setText(
                self.settings.value("/DigiAgriApp/user")
            )

        if self.settings.contains("/DigiAgriApp/password"):
            self.settings_dlg.password_lineEdit.setText(
                self.settings.value("/DigiAgriApp/password")
            )
        print("before _check_token")
        if self._check_token():
            self._get_farms()
        self.settings_dlg.show()

    def show_import_dialog(self):
        """Function to show import"""
        self._check_server()
        self._check_farm()
        if not self._working_server():
            self.import_dlg.close()
        action = self.import_dlg.sender()
        self.import_dlg.tableWidget.clear()
        self.layers_list = {}
        col = 0
        row = 0
        actual_rows = self.import_dlg.tableWidget.rowCount()
        addtype = None
        fields_to_add = ["Layer name", "Upload"]
        if action.text() == self.tr("Vector"):
            addtype = "vector"
            fields_to_add.extend(["DigiAgriApp feature type", "Selected features"])
        elif action.text() == self.tr("Raster"):
            addtype = "raster"
            fields_to_add.extend(["Field", "Time", "Sensor"])
        descr = f"Upload {addtype} layers to you DigiAgriApp server."
        descr += " You can select which layer upload with its metadata."
        self.import_dlg.label_descr.setText(descr)
        self.import_dlg.label_farm.setText(self.tr(f"<b>Selected farm is {self.farm}</b>"))
        if self.import_dlg.tableWidget.columnCount() < len(fields_to_add):
            while self.import_dlg.tableWidget.columnCount() != len(fields_to_add):
                self.import_dlg.tableWidget.insertColumn(col)
                col += 1
        elif self.import_dlg.tableWidget.columnCount() > len(fields_to_add):
            col_count = self.import_dlg.tableWidget.columnCount()
            while self.import_dlg.tableWidget.columnCount() != len(fields_to_add):
                self.import_dlg.tableWidget.removeColumn(col_count + 1)
                col_count -= 1

        self.import_dlg.tableWidget.setHorizontalHeaderLabels(fields_to_add)

        layer_type_count = 0
        for layer in QgsProject.instance().mapLayers().values():
            if isinstance(layer, QgsVectorLayer) and addtype == "vector":
                self.layers_list[layer.name()] = layer
                layer_type_count += 1
            elif isinstance(layer, QgsRasterLayer)  and addtype == "raster":
                self.layers_list[layer.name()] = layer
                layer_type_count += 1
        for layer in self.layers_list.values():
            if actual_rows == 0:
                self.import_dlg.tableWidget.insertRow(row)
            elif actual_rows != 0 and row >= actual_rows:
                self.import_dlg.tableWidget.insertRow(row)
            upload_checkbox = QTableWidgetItem()
            upload_checkbox.setFlags(
                Qt.ItemIsUserCheckable | Qt.ItemIsEnabled
            )
            upload_checkbox.setCheckState(Qt.Unchecked)
            self.import_dlg.tableWidget.setItem(
                row, 0, QTableWidgetItem(layer.name())
            )
            self.import_dlg.tableWidget.setItem(row, 1, upload_checkbox)

            if isinstance(layer, QgsVectorLayer) and addtype == "vector":
                digiagriapp_type = QComboBox()
                digiagriapp_type.addItems(
                    [
                        self.tr("field"),
                        self.tr("subfield"),
                        self.tr("row"),
                        self.tr("plant"),
                        self.tr("pole")
                    ]
                )
                self.import_dlg.tableWidget.setCellWidget(
                    row, 2, digiagriapp_type
                )
                features_checkbox = QTableWidgetItem()
                features_checkbox.setFlags(
                    Qt.ItemIsUserCheckable | Qt.ItemIsEnabled
                )
                features_checkbox.setCheckState(Qt.Unchecked)
                self.import_dlg.tableWidget.setItem(
                    row, 3, QTableWidgetItem(features_checkbox)
                )
            elif isinstance(layer, QgsRasterLayer)  and addtype == "raster":
                fields_combo = QComboBox()
                for fid, finame in self.fields_id.items():
                    fields_combo.addItem(finame, fid)
                self.import_dlg.tableWidget.setCellWidget(row, 2, fields_combo)
                timeItem = QLineEdit()
                self.import_dlg.tableWidget.setCellWidget(row, 3, timeItem)
                sensor_combo = QComboBox()
                self.import_dlg.tableWidget.setCellWidget(row, 4, sensor_combo)

            row += 1

        while self.import_dlg.tableWidget.rowCount() > len(self.layers_list):
            self.import_dlg.tableWidget.removeRow(
                self.import_dlg.tableWidget.rowCount()-1
            )
        self._refresh_table()
        self.import_dlg.exec()

    def import_data(self):
        """Run method that performs all the real work"""

        # Create the dialog with elements (after translation) and keep reference
        # Only create GUI ONCE in callback,
        # so that it will only load when the plugin is started

        # Run the dialog event loop
        self._check_farm()
        result = self.import_dlg.exec_()
        # See if OK was pressed
        if result:
            # Do something useful here - delete the line containing pass and
            # substitute with your code.
            for rownum in range(self.import_dlg.tableWidget.rowCount()):
                if self.import_dlg.tableWidget.item(rownum, 1).checkState() == 2:
                    layer_name = self.import_dlg.tableWidget.item(rownum, 0).text()
                    layer = self.layers_list[layer_name]
                    if isinstance(layer, QgsVectorLayer):
                        layer_digi_type = self.import_dlg.tableWidget.cellWidget(
                            rownum, 2
                        ).currentText()
                        if self.import_dlg.tableWidget.item(rownum, 3).checkState() == 2:
                            layer_allfeats = False
                        else:
                            layer_allfeats = True

                        import_url = FIELD_IMPORT.format(
                            server=self.server, typ=layer_digi_type
                        )
                        self._import_vector(layer, layer_allfeats, import_url)
                        filepath = layer.dataProvider().uri().uri().split('|')[0].strip()
                        QgsProject.instance().removeMapLayer(layer)

                        if layer_digi_type == "field":
                            self._export_field(filepath)
                            layer = self._reload_layer(filepath, "field")
                            self._set_single_style(layer, "#FFFF00")
                        elif layer_digi_type == "subfield":
                            self._export_subfield(filepath)
                            layer = self._reload_layer(filepath, "subfield")
                            self._set_single_style(layer, "#00FF00")
                        elif layer_digi_type == "row":
                            self._export_row(filepath)
                            layer = self._reload_layer(filepath, "row")
                            self._set_single_style(layer, "#8b4513")
                        elif layer_digi_type == "plant":
                            self._export_plant(filepath)
                            layer = self._reload_layer(filepath, "plant")
                            self._set_single_style(layer, "#006400")
                        elif layer_digi_type == "pole":
                            self._export_plant(filepath)
                            layer = self._reload_layer(filepath, "pole")
                            self._set_single_style(layer, "#808080")
                    elif isinstance(layer, QgsRasterLayer):
                        self.iface.messageBar().pushMessage(
                                self.tr("Warning"),
                                self.tr("Raster not yet implemented"),
                                level=Qgis.Warning,
                            )
            self.import_dlg.close()
        else:
            self.import_dlg.close()
        return True

    def _write_vector(self, layer, file_output):
        """ Write vector layer to file

        :param layer: Vector layer object.
        :type layer: obj, QgsVectorLayer

        :returns: Return True or False if vector is correctly written.
        :rtype: Boolean
        """
        save_options = QgsVectorFileWriter.SaveVectorOptions()
        if os.path.isfile(file_output):
            save_options.actionOnExistingFile = QgsVectorFileWriter.CreateOrOverwriteLayer
        save_options.layerName = layer.name()
        save_options.fileEncoding = layer.dataProvider().encoding()
        ##save_options.driverName = "GeoJSON"
        transform_context = QgsProject.instance().transformContext()
        error = QgsVectorFileWriter.writeAsVectorFormatV3(
            layer,
            file_output,
            transform_context,
            save_options,
        )
        if error[0] == QgsVectorFileWriter.NoError:
            self.iface.messageBar().pushMessage(
                self.tr("Success"),
                self.tr(f"Layer '{layer.name()}' correctly written to {file_output}"),
                level=Qgis.Success,
            )
            return True
        else:
            self.iface.messageBar().pushMessage(
                self.tr("Error"),
                self.tr(f"Layer '{layer.name()}' not written correctly: {error}"),
                level=Qgis.Success,
            )
            return False

    def show_create_plant_dialog(self):
        """Function to show the create plants dialog"""
        self._check_server()
        self._check_farm()
        self.plants_dlg.comboBoxField.clear()
        if not self._working_server():
            return False
        self._get_fields()
        for fieldid, fieldname in self.fields_id.items():
            self.plants_dlg.comboBoxField.addItem(fieldname, fieldid)
        self.plants_dlg.label_farm.setText(self.tr(f"<b>Selected farm is {self.farm}</b>"))
        if not self.material:
            self._get_material()
        self.plants_dlg.exec()

    def plant_creation(self):
        """Function to create plants"""
        file_in = self.plants_dlg.lineEditOutput.text()
        check_material = self.plants_dlg.material_checkBox.isChecked()
        check_poles = self.plants_dlg.poles_checkBox.isChecked()
        if not file_in:
            self.iface.messageBar().pushMessage(
                self.tr("Error"),
                self.tr("Please set the input CSV file path."),
                level=Qgis.Critical,
            )
            return False
        if os.path.exists(file_in) is False:
            self.iface.messageBar().pushMessage(
                self.tr("Error"),
                self.tr(f"The input CSV file path {file_in} does not exist."),
                level=Qgis.Critical,
            )
            return False
        fieldid = self.plants_dlg.comboBoxField.currentData()
        if not fieldid:
            self.iface.messageBar().pushMessage(
                self.tr("Error"),
                self.tr("Please set the field ID selecting a field"),
                level=Qgis.Critical,
            )
            return False
        subfieldid = self.plants_dlg.comboBoxSubfield.currentData()
        if not subfieldid:
            self.iface.messageBar().pushMessage(
                self.tr("Error"),
                self.tr("Please set the subfield ID selecting a subfield"),
                level=Qgis.Critical,
            )
            return False
        with open(file_in, "r") as csvfile:
            readlines = csvfile.readlines()

        if readlines[0].count("|") > 0:
            sep = "|"
        elif readlines[0].count(";") > 0:
            sep = ";"
        elif readlines[0].count(",") > 0:
            sep = ","
        else:
            self.iface.messageBar().pushMessage(
                self.tr("Error"),
                self.tr("CSV separator not recognized, please use one between '|', ';', ','"),
                level=Qgis.Critical,
            )
            return False
        headers = readlines[0].strip().split(sep)
        material_index = headers.index("material")
        row_index = headers.index("row")
        label_index = headers.index("label")
        theory_index = headers.index("theory")
        evidence_index = headers.index("evidence")
        self.data_import = {}
        old_row = None
        pole_number = 0
        plant_number = 0
        for line in readlines[1:]:
            infos = line.strip().split(sep)
            row_name = infos[row_index]
            if row_name not in self.data_import.keys():
                if old_row:
                    if plant_number > 0:
                        self.data_import[old_row]["plants_block"].append(plant_number)
                        self.data_import[old_row]["last"] = "plant"
                    else:
                        self.data_import[old_row]["last"] = "pole"
                    if "pole_number" not in self.data_import[old_row].keys() and pole_number > 0:
                        self.data_import[old_row]["pole_number"] = pole_number
                self.data_import[row_name] = {}
                self.data_import[row_name]["points"] = []
                self.data_import[row_name]["plants_block"] = []
                if infos[material_index] in ["", "p", "P"]:
                    self.data_import[row_name]["first"] = "pole"
                else:
                    self.data_import[row_name]["first"] = "plant"
                    self.data_import[row_name]["last"] = "pole"
                plant_number = 0
                pole_number = 0
                old_row = row_name
            if infos[material_index] in ["", "p", "P"]:
                print(f"plant number {plant_number}")
                if plant_number > 0:
                    self.data_import[row_name]["plants_block"].append(plant_number)
                pole_number += 1
                plant_number = 0
            else:
                plant_number += 1
                poi = {
                    "material": infos[material_index],
                    "label": infos[label_index],
                    "theory": infos[theory_index],
                    "evidence": infos[evidence_index]
                }
                if check_material:
                    if infos[material_index] in ["F", "f"]:
                        # pole is in the half of distance between two plants
                        try:
                            poi["spacing"] = self.data_import[row_name]["points"][-1]["spacing"] / 2
                        except:
                            poi["spacing"] = None
                        poi["multiplant"] = False
                    else:
                        mat = self.material.get(int(infos[material_index]))
                        try:
                            poi["spacing"] = mat["spacing_dict"]["plant"]
                            if mat["spacing_dict"]["multiplant"] in (True, 't'):
                                poi["multiplant"] = True
                            else:
                                poi["multiplant"] = False
                        except:
                            poi["spacing"] = None
                            poi["multiplant"] = False
                self.data_import[row_name]["points"].append(poi)
        if plant_number > 0:
            self.data_import[row_name]["plants_block"].append(plant_number)
            self.data_import[row_name]["last"] = "plant"
        else:
            self.data_import[row_name]["last"] = "pole"
        if "pole_number" not in self.data_import[old_row].keys():
            self.data_import[old_row]["pole_number"] = pole_number
        try:
            rowid = self.plants_dlg.comboBoxRow.currentData()
        except Exception:
            rowid = None
        if rowid:
            if subfieldid:
                url = SUBFIELD_ROW_URL.format(
                    server=self.server,
                    farmid=self.farm_id,
                    fieldid=fieldid,
                    subfieldid=subfieldid
                ) + f"{rowid}/"
            else:
                url = FIELD_ROW_URL.format(
                    server=self.server,
                    farmid=self.farm_id,
                    fieldid=fieldid
                ) + f"{rowid}/"
        elif subfieldid:
            url = SUBFIELD_URL.format(
                server=self.server,
                farmid=self.farm_id,
                fieldid=fieldid
            ) + f"{subfieldid}/rows/"
        with requests.Session() as ses:
            req = ses.get(
                url,
                headers={"Authorization": f"Token {self.token}"}
            )
            if not req.ok:
                reason = self._request_error(req.json())
                self.iface.messageBar().pushMessage(
                    self.tr("Warning"),
                    self.tr(
                        f"Problem to get subfield for farm with ID {self.farm_id} with subfield with ID {fieldid}; error: {reason}"
                    ),
                )
                return False
        rows = req.json()
        if isinstance(rows, list):
            plant_layers = []
            for row in rows:
                geom = LineString(row["geom"]["coordinates"])
                plant_layers.append(self._create_points(
                    geom,
                    self.data_import[row["name"]],
                    row["name"],
                    fieldid,
                    subfieldid,
                    row["id"],
                    check_poles
                ))
                # if len(outcoords) != len(self.data_import[row["name"]]["points"]):
                #     self.iface.messageBar().pushMessage(
                #         self.tr("Warning"),
                #         self.tr(
                #             "Number of plants from CSV is different from coordinates calculated"
                #         ),
                #     )
                # for x in range(len(outcoords)):
                #     self.data_import[row["name"]]["points"][x]["geom"] = outcoords[x]
        elif isinstance(rows, dict):
            geom = LineString(rows["geom"]["coordinates"])
            plant_layers = [self._create_points(
                geom,
                self.data_import[rows["name"]],
                rows["name"],
                fieldid,
                subfieldid,
                rows["id"],
                check_poles
            )]
            # if len(outcoords) != len(self.data_import[rows["name"]]["points"]):
            #     self.iface.messageBar().pushMessage(
            #         self.tr("Warning"),
            #         self.tr(
            #             "Number of plants from CSV is different from coordinates calculated"
            #         ),
            #     )
            # for x in range(len(outcoords)):
            #     self.data_import[rows["name"]]["points"][x]["geom"] = outcoords[x]
        canvas = self.iface.mapCanvas()
        if len(plant_layers) == 1:
            extent = plant_layers[0].extent()
        else:
            extent = None
            for layer in plant_layers:
                if extent is None:
                    extent = layer.extent()
                else:
                    extent.combineExtentWith(layer.extent())
        canvas.setExtent(extent)
        canvas.refresh()
        qd = QuestionDialog(parent=self.iface.mainWindow())
        if qd.exec():
            print(plant_layers)
            plant_url = PLANTS_CREATE.format(server=self.server)
            for layer in plant_layers:
                self._import_vector(layer, True, plant_url)
        else:
            print("Cancel!")

    def populate_subfields(self):
        self.plants_dlg.comboBoxSubfield.clear()
        self.plants_dlg.comboBoxSubfield.addItem("---", None)
        fieldid = self.plants_dlg.comboBoxField.currentData()

        subfields = self._get_subfields(fieldid)
        if not subfields:
            return False
        for subfield in subfields:
            self.plants_dlg.comboBoxSubfield.addItem(subfield["name"], subfield["id"])
        return True

    def populate_rows(self):
        self.plants_dlg.comboBoxRow.clear()
        self.plants_dlg.comboBoxRow.addItem("---", None)
        fieldid = self.plants_dlg.comboBoxField.currentData()
        subfieldid = self.plants_dlg.comboBoxSubfield.currentData()
        rows = self._get_rows(fieldid, subfieldid)
        if not rows:
            return False
        for row in rows:
            self.plants_dlg.comboBoxRow.addItem(row["name"], row["id"])
        return True

    def show_export_vector_dialog(self):
        """Function to show export vector data dialog"""
        # See if OK was pressed
        self._check_farm()
        self.export_vector_dlg.label_descr.setWordWrap(True)
        pdb.set_trace()
        if len(self.minmax_years) == 0:
            print("inside get material")
            self._get_material()
        pdb.set_trace()
        self.export_vector_dlg.comboBox_year.clear()
        self.export_vector_dlg.comboBox_year.addItems([str(i) for i in reversed(range(self.minmax_years[0], self.minmax_years[1] + 1))])
        self.export_vector_dlg.comboBox_year.setCurrentText(self.minmax_years[1])
        self.export_vector_dlg.label_farm.setText(self.tr(f"<b>Selected farm is {self.farm}</b>"))
        result = self.export_vector_dlg.exec()
        if result:
            self._check_server()
            self._check_farm()
            if not self._check_farm():
                self.iface.messageBar().pushMessage(
                    self.tr("Error"),
                    self.tr("Please set the farm in the settings window."),
                    level=Qgis.Critical,
                )
                return False
            file_output = self.export_vector_dlg.lineEditOutput.text()
            if not file_output:
                self.iface.messageBar().pushMessage(
                    self.tr("Error"),
                    self.tr("Please set the output file path."),
                    level=Qgis.Critical,
                )
                return False
            token_exist = self._check_token()
            if not token_exist:
                self.iface.messageBar().pushMessage(
                    self.tr("Error"),
                    self.tr("Impossible connect to the server, please check the settings."),
                    level=Qgis.Critical,
                )
                return False
            print(self.farm)
            fields_check = self.export_vector_dlg.fields_checkBox.isChecked()
            subfields_check = self.export_vector_dlg.subfields_checkBox.isChecked()
            rows_check = self.export_vector_dlg.rows_checkBox.isChecked()
            plants_check = self.export_vector_dlg.plants_checkBox.isChecked()
            poles_check = self.export_vector_dlg.poles_checkBox.isChecked()
            if fields_check is True or subfields_check is True or rows_check is True or plants_check is True or poles_check is True:
                self._get_fields()
            else:
                return True

            if len(self.fields) < 1:
                self.iface.messageBar().pushMessage(
                    self.tr("Warning"),
                    self.tr("There are no fields in the server"),
                )
                return False
            if not self.farm_stats:
                self.farm_stats = self._get_farm_stats()
            self._get_material(csv=True)

            if fields_check:
                self._export_field(file_output)
                self._reload_layer(file_output, "field")
                layer = QgsProject.instance().mapLayersByName(f"digiagriapp_field")[0]
                self._set_single_style(layer, "#FFFF00")
            if subfields_check:
                self._export_subfield(file_output)
                self._reload_layer(file_output, "subfield")
                layer = QgsProject.instance().mapLayersByName(f"digiagriapp_subfield")[0]
                self._set_single_style(layer, "#00FF00")
            if rows_check:
                self._export_row(file_output)
                self._reload_layer(file_output, "row")
                layer = QgsProject.instance().mapLayersByName(f"digiagriapp_row")[0]
                self._set_single_style(layer, "#8b4513")
            if plants_check:
                self._export_plant(file_output)
                self._reload_layer(file_output, "plant")
                layer = QgsProject.instance().mapLayersByName(f"digiagriapp_plant")[0]
                self._set_single_style(layer, "#006400")
            if poles_check:
                self._export_pole(file_output)
                self._reload_layer(file_output, "pole")
                layer = QgsProject.instance().mapLayersByName(f"digiagriapp_pole")[0]
                self._set_single_style(layer, "#808080")

            return True
        else:
            self.import_dlg.close()
        return True

    # def show_export_raster_dialog(self):
    #     """Function to export vector data"""
